<?php
declare(strict_types=1);

namespace Pinpoint\CarouselManager\Model\Carousel;

use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Serialize\Serializer\Json;
use Pinpoint\CarouselManager\Helper\Config;
use Pinpoint\CarouselManager\Model\ResourceModel\Carousel\CollectionFactory;
use Magento\Framework\Config\DataInterfaceFactory;
use Pinpoint\CarouselManager\Model\SlideCtaRepository;
use Pinpoint\CarouselManager\Model\SlideRepository;

class DataProvider extends \Magento\Ui\DataProvider\AbstractDataProvider
{
    const FORM_IDENTIFIER = 'pinpoint_carouselmanager_carousel_form';

    protected $collection;

    protected $dataPersistor;

    protected $loadedData;
    /**
     * @var DataInterfaceFactory
     */
    private $configFactory;
    /**
     * @var \Magento\Framework\Registry
     */
    private $registry;
    /**
     * @var SlideRepository
     */
    private $slideRepository;
    /**
     * @var \Magento\Backend\Model\UrlInterface
     */
    private $urlBuilder;
    /**
     * @var Config
     */
    private $configHelper;
    /**
     * @var Json
     */
    private $json;
    /**
     * @var SlideCtaRepository
     */
    private $slideCtaRepository;


    public function __construct(
        $name,
        $primaryFieldName,
        $requestFieldName,
        CollectionFactory $collectionFactory,
        DataPersistorInterface $dataPersistor,
        \Magento\Framework\Registry $registry,
        DataInterfaceFactory $configFactory,
        Json $jsonSerializer,
        SlideCtaRepository $ctaRepository,
        \Magento\Backend\Model\UrlInterface $urlBuilder,
        Config $configHelper,
        array $meta = [],
        array $data = []
    ) {
        $this->collection = $collectionFactory->create();
        $this->dataPersistor = $dataPersistor;
        $this->configFactory = $configFactory;
        $this->registry = $registry;
        $this->urlBuilder = $urlBuilder;
        $this->configHelper = $configHelper;
        $this->slideCtaRepository = $ctaRepository;
        $this->json = $jsonSerializer;
        parent::__construct($name, $primaryFieldName, $requestFieldName, $meta, $data);
    }

    public function getMeta()
    {
        $component = $this->configFactory->create(['componentName' => self::FORM_IDENTIFIER])->get(self::FORM_IDENTIFIER);

        $slides = $this->registry->registry('pinpoint_carouselmanager_carousel_slides');

        foreach ($slides as $slide) {
            $slideDemoCopy = $component['children']['slide_demo'];
            $slideDemoCopy['arguments']['data']['config']['label'] = $slide->getTitle();
//        $slideDemoCopy['arguments']['data']['config']['sortOrder'] = '500';
            if (isset($slideDemoCopy['arguments']['data']['js_config']['component'])) {
                $slideDemoCopy['arguments']['data']['config']['componentType'] = 'fieldset';
            }

            $slideDemoCopy['attributes']['name'] = $slide->getIdentifier();
            $slideDemoCopy = $this->initializeChildren($slideDemoCopy);

            //Initialize CTAs
//            for($i = 2; $i <= $this->configHelper->getMaximumNumberOfCtas(); $i++) {
            for ($i = 2; $i <= $slide->getCtaNumber(); $i++) {
                $ctaCopy = $slideDemoCopy['children']['cta_1'];
                $ctaCopy['arguments']['data']['config']['label'] = 'CTA ' . $i;
                $ctaCopy['arguments']['data']['config']['sortOrder'] = $i * 100;
                $ctaCopy['attributes']['name'] = $slide->getIdentifier();
                $ctaCopy = $this->initializeChildren($ctaCopy);
                $slideDemoCopy['children']['cta_' . $i] = $ctaCopy;
            }
            //End initialize CTAs
            $slideDemoCopy['children']['delete_slide'] = $this->getDeleteButton($slide->getId(), $slide->getCarouselId());
            $this->meta[$name = $slide->getIdentifier()] = $slideDemoCopy;
        }

        return $this->meta;
    }

    /**
     * Get data
     *
     * @return array
     */
    public function getData()
    {
        if (isset($this->loadedData)) {
            return $this->loadedData;
        }
        foreach ($this->collection->getItems() as $model) {
            $this->loadedData[$model->getId()]['general'] = $model->getData();
        }

        $carousel = $this->registry->registry('pinpoint_carouselmanager_carousel');
        $slides = $this->registry->registry('pinpoint_carouselmanager_carousel_slides');

        foreach ($slides as $slide) {
            $this->loadedData[$carousel->getId()][$slide->getIdentifier()] = $this->getSlideData($slide);
            $slideCtas = $this->slideCtaRepository->getBySlideId($slide->getId());

            $i = 1;
            foreach ($slideCtas as $slideCta) {
                $this->loadedData[$carousel->getId()][$slide->getIdentifier()]['cta_' . $i++] = $slideCta->getData();
            }
        }
        
        return $this->loadedData;
    }

    private function getComponentType($component)
    {
        $ar = explode('\\', $component);
        $class = array_pop($ar);
        return lcfirst($class);
    }

    private function getDeleteButton($slideId, $carouselId)
    {
        $deleteSlideUrl = $this->urlBuilder->getUrl('pinpoint_carouselmanager/slide/delete',
            ['slide_id' => $slideId, 'carousel_id' => $carouselId]);

        return [
            'arguments' => [
                'data' => [
                    'config' => [
                        'component' => 'Pinpoint_CarouselManager/js/form/components/delete-button',
                        'title' => 'Delete Slide',
                        'displayAsLink' => false,
                        'buttonClasses' => 'delete-slide-button',
                        'actions' => [
                            'params' => [
                                'slide_id' => $slideId,
                                'delete_slide_url' => $deleteSlideUrl
                            ]
                        ],
                        'componentType' => 'container'
                    ]
                ]
            ],
            'attributes' => [
                'class' => 'Magento\Ui\Component\Container',
                'component' => 'Pinpoint_CarouselManager/js/form/components/delete-button',
                'name' => 'delete_slide'
            ],
        ];
    }

    private function initializeChildren($slideDemoCopy)
    {
        foreach ($slideDemoCopy['children'] as &$element) {
            $element['arguments']['data']['config']['componentType'] = $this->getComponentType($element['attributes']['class']);
            if (isset($element['children']) && is_array($element['children'])) {
                foreach ($element['children'] as &$child) {
                    $child['arguments']['data']['config']['componentType'] = $this->getComponentType($child['attributes']['class']);
                }
            }
        }

        return $slideDemoCopy;
    }

    /**
     * @param $slide
     * @return array|null
     */
    protected function getImageDetails($image)
    {
        if ($image) {
            $url = $this->urlBuilder->getBaseUrl(['_type' =>\Magento\Framework\UrlInterface::URL_TYPE_MEDIA]) . 'content/carousel/' . $image;
            $type = pathinfo($image, PATHINFO_EXTENSION);
            return [['name' => $image, 'type' => 'image/'.$type, 'size' => 0, 'url' => $url]];
        } else {
            return null;
        }
    }

    private function getSlideData($slide)
    {
        $slideData = $slide->getData();
        $slideData['background_image'] = $this->getImageDetails($slide->getBackgroundImage());
        $slideData['mobile_background_image'] = $this->getImageDetails($slide->getMobileBackgroundImage());
        if ($slide->getPaddingsAndMargins()) {
            $slideData['margins_and_padding'] = $this->json->unserialize($slide->getPaddingsAndMargins());
        }
        if ($slide->getMobilePaddingsAndMargins()) {
            $slideData['mobile_margins_and_padding'] = $this->json->unserialize($slide->getMobilePaddingsAndMargins());
        }

        return $slideData;
    }
}